﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Threading;
using Win32.TokenPrivileges;

namespace ConsoleApp_test
{
    class Program
    {
        [StructLayout(LayoutKind.Sequential)]
        public struct SYSTEMTIME
        {
            public short wYear;
            public short wMonth;
            public short wDayOfWeek;
            public short wDay;
            public short wHour;
            public short wMinute;
            public short wSecond;
            public short wMilliseconds;
        }

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool SetSystemTime(ref SYSTEMTIME st);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool SetLocalTime(ref SYSTEMTIME st);

        static void Main(string[] args)
        {
            var currentProcess = Process.GetCurrentProcess();
            Console.WriteLine(currentProcess.Id);

            while (true)
            {
                string responseText = null;

                try
                {
                    using (var client = new TcpClient("time.nist.gov", 13))
                    using (var streamReader = new StreamReader(client.GetStream()))
                    {
                        Thread.Sleep(5000);

                        //인터넷 시간 불러오기
                        responseText = streamReader.ReadToEnd();
                        var utcDateTimeString = responseText.Substring(7, 17);

                        if (DateTime.TryParseExact(utcDateTimeString, "yy-MM-dd HH:mm:ss", CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal
                        , out DateTime utcDateTime) == false)
                        {
                            Console.WriteLine(responseText);
                            continue;
                        }

                        var localDateTime = utcDateTime.ToLocalTime();

                        //pc에 적용하기
                        SYSTEMTIME st = new SYSTEMTIME();
                        st.wYear = (short)localDateTime.Year;
                        st.wMonth = (short)localDateTime.Month;
                        st.wDay = (short)localDateTime.Day;
                        st.wHour = (short)localDateTime.Hour;
                        st.wMinute = (short)localDateTime.Minute;
                        st.wSecond = (short)localDateTime.Second;
                        st.wMilliseconds = (short)localDateTime.Millisecond;

                        using (new AdjustPrivilege(PrivilegeName.SeSystemtimePrivilege))
                        {
                            Console.WriteLine("Privileges should now be granted.");
                            Console.WriteLine($"SeSystemtimePrivilege: {PrivilegeProvider.HasPrivilege(null, currentProcess, PrivilegeName.SeSystemtimePrivilege)}");

                            bool result = SetLocalTime(ref st);
                            if (result == false)
                            {
                                int lastError = Marshal.GetLastWin32Error();
                                Console.WriteLine(lastError);
                            }
                        }

                        Console.WriteLine($"Response: {responseText}, DateTime: {localDateTime}");
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("Exception: " + e.Message + "\n(" + responseText + ")");
                }
            }
        }
    }
}
